######## fastq_function.R

filter_fastq_loose<-function(fastqR1,fastqR2,fastqFolder,filterFolder){
  print("filter_fastq_loose")
  passQualityRead1 <- srFilter(function(x) {
  
    rowSums(as(quality(x), "matrix")>20)>135
  }, name="GoodQualityBases1")
  
  
  readsR1<-readFastq(paste0(fastqFolder,"/",fastqR1)) 
  readsR2<-readFastq(paste0(fastqFolder,"/",fastqR2))
  
  allReadsNum<-length(sread(readsR1))
  
  filter1<-passQualityRead1(readsR1)@.Data
  filter2<-passQualityRead1(readsR2)@.Data
  
  readsR1<-trimEnds(readsR1, "4", relation="<=")
  readsR2<-trimEnds(readsR2, "4", relation="<=")
  filter3<-nchar(sread(readsR1))>100
  filter4<-nchar(sread(readsR2))>100

  cut20R1<-length(which(filter1))
  cut20R2<-length(which(filter2))
  
  filterAll<-filter1&filter2&filter3&filter4
  
  readsR1<-readsR1[filterAll]
  readsR2<-readsR2[filterAll]
  

  
  filteredReads<-length(which(filterAll))
  
  filterRatio<-filteredReads/allReadsNum
  
  writeFastq(readsR1, paste0(filterFolder,"/filter_",fastqR1,".fq"), 
             mode="w", full=FALSE, compress=FALSE)
  
  writeFastq(readsR2, paste0(filterFolder,"/filter_",fastqR2,".fq"), 
             mode="w", full=FALSE, compress=FALSE)
  
  c(fastqR1,allReadsNum,filteredReads,filterRatio,cut20R1,cut20R2)
}



filter_fastq<-function(fastqR1,fastqR2,fastqFolder,filterFolder){
  
  passQualityRead1 <- srFilter(function(x) {
    apply(as(quality(x), "matrix"),1,min,na.rm = TRUE)>19
  }, name="GoodQualityBases1")
  
  
  readsR1<-readFastq(paste0(fastqFolder,"/",fastqR1))
  readsR1<-trimEnds(readsR1, "?", relation="<=")
  
  readsR2<-readFastq(paste0(fastqFolder,"/",fastqR2))
  readsR2<-trimEnds(readsR2, "?", relation="<=")
  
  
  allReadsNum<-length(sread(readsR1))
  
  qualMatrix<-as(quality(readsR1), "matrix")
  qualMatrix2<-as(quality(readsR2), "matrix")
  
  filter1<-passQualityRead1(readsR1)@.Data
  filter2<-passQualityRead1(readsR2)@.Data
  filter3<-nchar(sread(readsR1))>100
  filter4<-nchar(sread(readsR2))>100
  
  cut20R1<-length(which(filter1))
  cut20R2<-length(which(filter2))
  
  cut30R1<-length(which(apply(qualMatrix,1,min,na.rm = TRUE)>29))
  cut30R2<-length(which(apply(qualMatrix2,1,min,na.rm = TRUE)>29))
  
  
  filterAll<-filter1&filter2&filter3&filter4
  
  readsR1<-readsR1[filterAll]
  readsR2<-readsR2[filterAll]
  
  filteredReads<-length(which(filterAll))
  
  filterRatio<-filteredReads/allReadsNum
  
  writeFastq(readsR1, paste0(filterFolder,"/filter_",fastqR1,".fq"), 
             mode="w", full=FALSE, compress=FALSE)
  
  writeFastq(readsR2, paste0(filterFolder,"/filter_",fastqR2,".fq"), 
             mode="w", full=FALSE, compress=FALSE)
  
  c(fastqR1,allReadsNum,filteredReads,filterRatio,cut20R1,cut30R1,cut20R2,cut30R2)
}



read_filter_high<-function(fastqR1,fastqR2,fastqFolder){
  
  readsR1<-readFastq(paste0(fastqFolder,"/",fastqR1))
  allReadsNum<-length(sread(readsR1))
  readsR1<-trimEnds(readsR1, "I", relation="<=")
  
  readsR2<-readFastq(paste0(fastqFolder,"/",fastqR2))
  readsR2<-trimEnds(readsR2, "I", relation="<=")
  
  filter1_len<-nchar(sread(readsR1))>100
  filter2_len<-nchar(sread(readsR2))>100
  
  qualMatrix<-as(quality(readsR1), "matrix")
  qualMatrix2<-as(quality(readsR2), "matrix")
  
  filter1_30<-apply(qualMatrix,1,min,na.rm = TRUE)>29
  filter2_30<-apply(qualMatrix2,1,min,na.rm = TRUE)>29
  
  filter1_40<-apply(qualMatrix,1,min,na.rm = TRUE)>39
  filter2_40<-apply(qualMatrix2,1,min,na.rm = TRUE)>39
  
  writeFastq(readsR1[filter1_30&filter1_len], paste0(filterFolder,"/filter_30_",fastqR1,".fq"), 
             mode="w", full=FALSE, compress=FALSE)
  writeFastq(readsR1[filter1_40&filter1_len], paste0(filterFolder,"/filter_40_",fastqR1,".fq"), 
             mode="w", full=FALSE, compress=FALSE)
  
  writeFastq(readsR2[filter2_30&filter2_len], paste0(filterFolder,"/filter_30_",fastqR2,".fq"), 
             mode="w", full=FALSE, compress=FALSE)
  writeFastq(readsR2[filter2_40&filter2_len], paste0(filterFolder,"/filter_40_",fastqR2,".fq"), 
             mode="w", full=FALSE, compress=FALSE)	
  
  c(fastqR1,allReadsNum,length(filter1_30&filter1_len),length(filter1_40&filter1_len)
    ,length(filter2_30&filter2_len),length(filter2_40&filter2_len))
  
  
}