### result_base_function.R
norm_0_1<-function(x){
  (x-min(x))/(max(x)-min(x))
}

sem<-function(x){
  sd(x)/sqrt(length(x))
}

cv<-function(x){
  sd(x)/mean(x)
}

codon_freq_p<-function(rep1,rep2){

  subs_codon_exp_freq<-(rep1+rep2)/2
  
  codon_list<-rownames(subs_codon_exp_freq)
  
  codon_exp_freq_info<-NULL
  for(aa in 1:ncol(subs_codon_exp_freq)){
  
  tmp_info<-cbind(rep(aa,length(codon_list)),codon_list,subs_codon_exp_freq[,aa],
  rep1[,aa],rep2[,aa])
  codon_exp_freq_info<-rbind(codon_exp_freq_info,tmp_info)
  }
  
  subs_codon_exp_sem<-as.matrix(unlist(apply(codon_exp_freq_info[,4:5],1,sem)))

  log_exp_freq<-log10(as.numeric(codon_exp_freq_info[,3])+1e-08)
  log_exp_p<-pnorm(-scale(log_exp_freq))
  codon_exp_freq_info<-cbind(codon_exp_freq_info,log_exp_freq,log_exp_p,subs_codon_exp_sem)
  colnames(codon_exp_freq_info)<-c("aa_pos","codon","codon_freq","rep1_freq","rep2_freq","log_freq","p_value","SEM")
  
  codon_exp_freq_info

}

### calculate deletion strength for individual aa
get_del_sum<-function(aa_del_matrix,non_del){
  
  sum_del<-rep(0,nrow(aa_del_matrix))
  
  for(aa in 1:nrow(aa_del_matrix)){
    aa_pos<-aa
    del_length<-which(aa_del_matrix[aa,]>non_del)
    if(length(del_length)>0){
      for(dd in 1:length(del_length)){
        ### only compute mutiple aa deletion
        if(del_length[dd]>1){
          sum_del[c(aa:(aa+del_length[dd]-1))]<-sum_del[c(aa:(aa+del_length[dd]-1))]+
            rep(aa_del_matrix[aa,del_length[dd]]/del_length[dd]^2,del_length[dd])
        }
        ### compute 3nt deletion related two aa
        if(del_length[dd]==1){
          sum_del[aa:(aa+1)]<-sum_del[c(aa:(aa+1))]+aa_del_matrix[aa,1]
          
        }
      }
    }
  }
  sum_del
}


get_integrated_del_sum<-function(aa_del_matrix,non_del){
  
  sum_del<-rep(0,nrow(aa_del_matrix))
  
  for(aa in 1:nrow(aa_del_matrix)){
    aa_pos<-aa
    del_length<-which(aa_del_matrix[aa,]>non_del)
    if(length(del_length)>0){
      for(dd in 1:length(del_length)){
	  
	    sum_del[c(aa:(aa+del_length[dd]-1))]<-sum_del[c(aa:(aa+del_length[dd]-1))]+
		rep(aa_del_matrix[aa,del_length[dd]]/del_length[dd]^2,del_length[dd])

      }
    }
  }
  sum_del
}

get_shifted_del_sum<-function(aa_del_matrix,non_del){
  
  sum_del<-rep(0,nrow(aa_del_matrix))
  
  for(aa in 1:nrow(aa_del_matrix)){
    aa_pos<-aa
    del_length<-which(aa_del_matrix[aa,]>non_del)
    if(length(del_length)>0){
      for(dd in 1:length(del_length)){
	  
	    sum_del[c(aa:(aa+del_length[dd]))]<-sum_del[c(aa:(aa+del_length[dd]))]+
		rep(aa_del_matrix[aa,del_length[dd]]/(del_length[dd])^2,(del_length[dd]+1))

      }
    }
  }
  sum_del
}




get_subs_aa_vector<-function(lib_subs){
  
  lib_subs_aa<-c(lib_subs)
  
  pos_aa<-character(length(lib_subs_aa)*21)
  
  aa_index<-rownames(lib_subs)
  
  pos_aa_fc<-rep(1,length(lib_subs_aa)*21)
  
  for(i in 1:length(lib_subs_aa)){
    
    pos_aa[((i-1)*21+1):(i*21)]<-paste0(i,".",aa_index)
    pos_aa_fc[((i-1)*21+1):(i*21)]<-c(unlist(lib_subs_aa[i]))
    
  }
  names(pos_aa_fc)<-pos_aa
  pos_aa_fc
}


count_matrix_cut<-function(matrix,th){
  
  for(rr in 1:nrow(matrix)){
    matrix[rr,which(matrix[rr,]<th)]<-0
    
  }
  matrix
}


subs_count_cut_freq<-function(folder,label,th,ref_path){


  
  codon_depth<-as.matrix(read.table(paste0(folder,"/",label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  
  subs_file<-read.table(paste0(folder,"/",label,".subs.count.codon.csv"),header=T,sep=",",dec=".")
  
  rownames(subs_file)<-as.character(subs_file[,1])
  subs_file<-subs_file[,-1]
  
  subs_matrix<-count_matrix_cut(subs_file,th)
  
  
  codon_depth<-as.matrix(read.table(paste0(folder,"/",label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  subs_freq<-sweep(data.frame(subs_matrix), 2, as.matrix(codon_depth), FUN = '/')
  
  rownames(subs_freq)<-rownames(subs_file)
  subs_freq
  
}


construct_del_matrix<-function(del_file,aa_len){
#print("construct_del_matrix")


  perfect_del_matrix<-shift_del_matrix<-matrix(0,ncol=10,nrow=aa_len+1)
  
  #single_aa_del<-rep(0,aa_len+1)
  
  for(dd in 1:nrow(del_file)){
    del_info<-del_file[dd,]
	
	del_start<-as.numeric(del_info$del_start)
    del_len<-as.numeric(del_info$del_length)
    del_freq<-as.numeric(del_info$del_freq)
    
    if(del_info$perfect_del==1){
      
      perfect_del_matrix[del_start,del_len]<-del_freq
      
    }else{

      shift_del_matrix[del_start,del_len]<-del_freq
    }
    
  }
  
  cbind(perfect_del_matrix,shift_del_matrix)
  
}



construct_del_matrix_single<-function(del_file,aa_len){
  del_matrix_exp<-matrix(0,ncol=10,nrow=aa_len+1)
  
  single_aa_del<-rep(0,aa_len+1)
  
  for(dd in 1:nrow(del_file)){
    del_info<-del_file[dd,]
    
    if(del_info$single_flag==1){
      
      single_aa_del[as.numeric(del_info$del_start)]<-as.numeric(del_info$del_freq)
      
    }else{
      
      del_start<-as.numeric(del_info$del_start)
      del_len<-as.numeric(del_info$del_length)
      del_freq<-as.numeric(del_info$del_freq)
      
      del_matrix_exp[del_start,del_len]<-del_freq
    }
    

    
  }
  
  cbind(del_matrix_exp,single_aa_del)
  
}




convert_nt_to_codon_single<-function(inframe_del,cds_start,cds_end){

  inframe_del_start<-inframe_del[,1]
  cds_del<-which(inframe_del_start>cds_start & 
                   inframe_del_start<cds_end)
  inframe_del<-inframe_del[cds_del,]

  ######
  ### inframe_del_start from 1 !! cds_start is pos 0
  tmp<-which((inframe_del[,1]-cds_start-1)%%3==0 & inframe_del[,2]==3)
  single_aa_del<-cbind((inframe_del[tmp,1]-cds_start-1)/3+1,inframe_del[tmp,2]/3,inframe_del[tmp,3])
  single_aa_del<-cbind(single_aa_del,rep(1,nrow(single_aa_del)))
  ######
  inframe_del<-inframe_del[-tmp,]
  inframe_del_start<-inframe_del[,1]
  inframe_del_length<-inframe_del[,2]
  inframe_del_count<-inframe_del[,3]
  

  aa_del_start<-floor((inframe_del_start-cds_start-1)/3)+1
  aa_del_length<-inframe_del_length/3

  del_codon<-cbind(aa_del_start,aa_del_length,inframe_del_count)
  
  multiple_del<-cbind(ddply(data.frame(del_codon), c("aa_del_start","aa_del_length"), summarise, count = sum(inframe_del_count)),0)
  colnames(multiple_del)<-c("aa_del_start","aa_del_length","aa_del_count","single_flag")
  colnames(single_aa_del)<-c("aa_del_start","aa_del_length","aa_del_count","single_flag")
  rbind(single_aa_del,multiple_del)

}


convert_nt_to_codon<-function(inframe_del,cds_start,cds_end){
#print("convert_nt_to_codon")
  inframe_del_start<-inframe_del[,1]
  cds_del<-which(inframe_del_start>cds_start & 
                   inframe_del_start<cds_end)
  inframe_del<-inframe_del[cds_del,]

  ######
  ### inframe_del_start from 1 !! cds_start is pos 0
  tmp<-which((inframe_del[,1]-cds_start-1)%%3==0)
  
  if(length(tmp)==0){
  
	  print("all combo")
	  
	  inframe_del_start<-inframe_del[,1]
	  inframe_del_length<-inframe_del[,2]
	  inframe_del_count<-inframe_del[,3]

	  aa_del_start<-floor((inframe_del_start-cds_start-1)/3)+1
	  aa_del_length<-inframe_del_length/3
	  
	  #print(inframe_del_count)
	  del_codon<-cbind(aa_del_start,aa_del_length,inframe_del_count)
print(del_codon)
	  shift_del<-cbind(ddply(data.frame(del_codon), c("aa_del_start","aa_del_length"), summarise, count = sum(inframe_del_count)),0)
	  

	  colnames(shift_del)<-c("aa_del_start","aa_del_length","aa_del_count","perfect_del")
	  
		shift_del

  }else if(length(tmp)==nrow(inframe_del)){
  
	  print("all deletion")
	  
		  perfect_aa_del<-cbind((inframe_del[tmp,1]-cds_start-1)/3+1,inframe_del[tmp,2]/3,inframe_del[tmp,3])
	  
	  perfect_aa_del<-cbind(perfect_aa_del,rep(1,nrow(perfect_aa_del)))

	  colnames(perfect_aa_del)<-c("aa_del_start","aa_del_length","aa_del_count","perfect_del")
	  
	  perfect_aa_del


  
  }else{
		perfect_aa_del<-cbind((inframe_del[tmp,1]-cds_start-1)/3+1,inframe_del[tmp,2]/3,inframe_del[tmp,3])
	  
	  perfect_aa_del<-cbind(perfect_aa_del,rep(1,nrow(perfect_aa_del)))
	  #print(perfect_aa_del)
	  inframe_del<-inframe_del[-tmp,]
	  
	  inframe_del_start<-inframe_del[,1]
	  inframe_del_length<-inframe_del[,2]
	  inframe_del_count<-inframe_del[,3]

	  aa_del_start<-floor((inframe_del_start-cds_start-1)/3)+1
	  aa_del_length<-inframe_del_length/3
	  
	  #print(inframe_del_count)
	  del_codon<-cbind(aa_del_start,aa_del_length,inframe_del_count)

	  shift_del<-cbind(ddply(data.frame(del_codon), c("aa_del_start","aa_del_length"), summarise, count = sum(inframe_del_count)),0)
	  

	  colnames(shift_del)<-c("aa_del_start","aa_del_length","aa_del_count","perfect_del")
	  colnames(perfect_aa_del)<-c("aa_del_start","aa_del_length","aa_del_count","perfect_del")
	  rbind(perfect_aa_del,shift_del)
  }

  ######
  

}






del_freq_from_count<-function(del_matrix,codon_depth){
  
  aa_del_freq<-matrix(0,ncol=4,nrow=nrow(del_matrix))
  
  for(dd in 1:nrow(del_matrix)){
    del_start<-as.numeric(del_matrix[dd,1])
    del_length<-as.numeric(del_matrix[dd,2])
    del_count<-as.numeric(del_matrix[dd,3])
    
    if((del_start+del_length)>length(codon_depth)){
      mean_coverage<-mean(codon_depth[c(del_start:length(codon_depth))])
    }else{
      mean_coverage<-mean(codon_depth[c(del_start:(del_start+del_length))])
      
    }
    
    del_freq<-del_count/mean_coverage
    
    aa_del_freq[dd,]<-c(del_start,del_length,del_count,del_freq)
    
    
  }
  aa_del_freq<-cbind(aa_del_freq,del_matrix[,4])  #colnames(aa_del_freq)<-c("del_start","del_length","del_count","del_freq","single_flag")
  colnames(aa_del_freq)<-c("del_start","del_length","del_count","del_freq","perfect_del")
  data.frame(aa_del_freq)
}
  


get_codon_given_AA<-function(aa){
  slc<-c("G","A","V","L","I","M","F","W","P","S","T","C","Y","N","Q","D","E","K","R","H","Stop")
  codon<-c(
  "GGT, GGC, GGA, GGG",  
  "GCT, GCC, GCA, GCG",
  "GTT, GTC, GTA, GTG",
  "CTT, CTC, CTA, CTG, TTA, TTG",
  "ATT, ATC, ATA",
  "ATG",
  "TTT, TTC",
  "TGG",
  "CCT, CCC, CCA, CCG",
  "TCT, TCC, TCA, TCG, AGT, AGC",
  "ACT, ACC, ACA, ACG",    
  "TGT, TGC",
  "TAT, TAC",
  "AAT, AAC",
  "CAA, CAG", 
  "GAT, GAC",
  "GAA, GAG",
  "AAA, AAG",
  "CGT, CGC, CGA, CGG, AGA, AGG",
  "CAT, CAC",
  "TAA, TAG, TGA")
  
  index<-which(slc==aa)
  codon[index]

  
}



synonymous_aa_info<-function(exp_codon_file,ref_codon_file,aa_type_freq_cut,subsize,WT_aa){

	ess_aa_type<-aa_type_freq_cut[,subsize]
	
	if(file.exists(paste0(tableFolder,"/",gene_label,"_exp_synonymous_mut.csv"))) file.remove(paste0(tableFolder,"/",gene_label,"_exp_synonymous_mut.csv"))
	if(file.exists(paste0(tableFolder,"/",gene_label,"_ref_synonymous_mut.csv"))) file.remove(paste0(tableFolder,"/",gene_label,"_ref_synonymous_mut.csv"))

	for(ss in 1:length(subsize)){
	  aa_ess_pos<-subsize[ss]
	  aa_ess_type<-WT_aa[aa_ess_pos]
	  
	  if(ess_aa_type[aa_ess_type,ss]>0){
		codon_list<-get_codon_given_AA(aa_ess_type)
		codon<-strsplit(codon_list,", ")[[1]]
		tmp_index<-which(as.numeric(exp_codon_file[,1])==aa_ess_pos & exp_codon_file[,2]%in%codon)
		exp_codon_info<-exp_codon_file[tmp_index,]
		ref_codon_info<-ref_codon_file[tmp_index,]
		
		t_test<-rep(0,length(tmp_index))
		
		for(tt in 1:length(t_test)){
		#print(as.numeric(exp_codon_file[tmp_index[tt],4:5]))
		t_test[tt]<-t.test(as.numeric(exp_codon_file[tmp_index[tt],4:5]),
			as.numeric(ref_codon_file[tmp_index[tt],4:5]),alternative="greater")$p.value
		}
		
		exp_codon_info[,1]<-paste0(aa_ess_type,exp_codon_info[,1])
		ref_codon_info[,1]<-paste0(aa_ess_type,ref_codon_info[,1])
		
		exp_codon_info<-cbind(exp_codon_info,t_test)
		write.table(exp_codon_info, 
					paste0(tableFolder,"/",gene_label,"_",aa_ess_pos,aa_ess_type,"_exp.csv"), 
					append = FALSE, quote = FALSE, 
					sep = ",",col.names = NA,
					row.names = TRUE) 
		
		write.table(ref_codon_info, 
					paste0(tableFolder,"/",gene_label,"_",aa_ess_pos,aa_ess_type,"_ref.csv"), 
					append = FALSE, quote = FALSE, 
					sep = ",",col.names = NA,
					row.names = TRUE) 
					
		write.table(exp_codon_info, 
					paste0(tableFolder,"/",gene_label,"_exp_synonymous_mut.csv"), 
					append = TRUE, quote = FALSE, 
					sep = ",",col.names = NA,
					row.names = TRUE) 	

		write.table(ref_codon_info, 
					paste0(tableFolder,"/",gene_label,"_ref_synonymous_mut.csv"), 
					append = TRUE, quote = FALSE, 
					sep = ",",col.names = NA,
					row.names = TRUE) 					
		
	  }
	  
	}


}


