####### result_function.R
library(plyr)
library("plotrix")
library("plot.matrix")
library(raster)  
library(plot3D)  

output_subs_for_subsampling<-function(folder,cds_start,cds_end,count_th,exp1_label,exp2_label,ref1_label,ref2_label,gene_label){
  
  figFolder<-paste0("results_fig_",folder)
  tableFolder<-paste0("result_output_",folder)
  
  aa_seq<-as.character(unlist(read.table(paste0("aa_seq/",gene_label,"_aa.txt"),header=F,sep=",",fill=TRUE,dec=".")))

  aa_len<-nchar(aa_seq)
    print(aa_len)
  
  subs_exp1_freq<-subs_count_cut_freq(gene_label,exp1_label,count_th) 
  subs_exp2_freq<-subs_count_cut_freq(gene_label,exp2_label,count_th)
 
  subs_ref1_freq<-subs_count_cut_freq(gene_label,ref1_label,count_th)  
  subs_ref2_freq<-subs_count_cut_freq(gene_label,ref2_label,count_th)

  
  ### remove SNP 
  SNP_cut<-0.1
  tmp<-which(subs_ref1_freq>SNP_cut & subs_ref2_freq>SNP_cut)
  if(length(tmp)>0){
  print("SNP")
  print(tmp)
	  subs_exp1_freq[subs_ref1_freq>SNP_cut & subs_ref2_freq>SNP_cut]<-0
	  subs_exp2_freq[subs_ref1_freq>SNP_cut & subs_ref2_freq>SNP_cut]<-0
	  subs_ref1_freq[subs_ref1_freq>SNP_cut & subs_ref2_freq>SNP_cut]<-0
	  subs_ref2_freq[subs_ref1_freq>SNP_cut & subs_ref2_freq>SNP_cut]<-0
  }
  
  ####### for synonymous substitution #######

  codon_exp_freq_info<-codon_freq_p(subs_exp1_freq,subs_exp2_freq)
    write.table(codon_exp_freq_info, 
              paste0(tableFolder,"/",gene_label,"_codon_exp_freq_info.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = F,col.names = T) 
			  
  codon_ref_freq_info<-codon_freq_p(subs_ref1_freq,subs_ref2_freq)
    write.table(codon_ref_freq_info, 
              paste0(tableFolder,"/",gene_label,"_codon_ref_freq_info.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = F,col.names = T) 			  

    ####### for synonymous substitution over #######
	
	
  aa_subs_exp1_freq<-codonToAA(subs_exp1_freq)
  aa_subs_exp2_freq<-codonToAA(subs_exp2_freq)
  aa_subs_ref1_freq<-codonToAA(subs_ref1_freq)
  aa_subs_ref2_freq<-codonToAA(subs_ref2_freq)

  exp1_sum_freq<-colSums(aa_subs_exp1_freq)
  exp2_sum_freq<-colSums(aa_subs_exp2_freq)
  
  ref1_sum_freq<-colSums(aa_subs_ref1_freq)
  ref2_sum_freq<-colSums(aa_subs_ref2_freq)
  

  aa_ref_freq<-(aa_subs_ref1_freq+aa_subs_ref2_freq)/2
  
  aa_subs_exp_freq<-(aa_subs_exp1_freq+aa_subs_exp2_freq)/2
  
  write.table(aa_subs_exp_freq, 
              paste0(tableFolder,"/",gene_label,"_subs_aa_freq.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = TRUE,col.names = NA) 
 

  exp_aa_t_test<-rep(1,length(exp1_sum_freq))
  
  exp1_sum_log_freq<-log10(exp1_sum_freq+1e-05)
  exp2_sum_log_freq<-log10(exp2_sum_freq+1e-05)
  
  ref1_sum_log_freq<-log10(ref1_sum_freq+1e-05)
  ref2_sum_log_freq<-log10(ref2_sum_freq+1e-05)
  
  avg_sum_freq<-(exp1_sum_freq+exp2_sum_freq)/2
  
  exp_sum_cv<-as.matrix(unlist(apply(cbind(exp1_sum_freq,exp2_sum_freq),1,cv)))
  #summary(avg_sum_freq)
  
  ##  calculate p-value and essential score of substitution
  
  set.seed(90)
  neg_dist<-rnorm(100)
  
  exp1_sum_freq_z<-scale(exp1_sum_freq)
  exp2_sum_freq_z<-scale(exp2_sum_freq)  
  
  subs_freq_p_value<-rep(1,length(exp1_sum_freq_z))
  for(zz in 1:length(exp1_sum_freq_z)){
    
	subs_freq_p_value[zz]<-wilcox.test(x=c(exp1_sum_freq_z[zz],exp2_sum_freq_z[zz]),y=neg_dist, alternative ="greater")$p.value
  
  }

  #subs_freq_p_value<-pnorm(-scale(avg_sum_freq))
  
  #print(subs_freq_p_value)
  subs_freq_score<-(-log10(subs_freq_p_value+1e-05))+log10(avg_sum_freq+1e-05)+10
  #is.na(subs_freq_score)==0
  subs_freq_score<-(subs_freq_score-min(subs_freq_score))/(max(subs_freq_score)-min(subs_freq_score))
  
  subs_classification<-rep(0,length(avg_sum_freq))
  
  ## cutsubstituion freq 
  #subs_p_cut<-0.05
  avg_freq_log<-log10(avg_sum_freq+1e-05)
  subs_cut<-mean(avg_freq_log)+3*sd(avg_freq_log)

  
  
  ########
  
  outliers<-which(avg_freq_log>subs_cut)
  
  if(length(outliers)>0){
  
  print(outliers)
  
	subs_classification[outliers]<-1
	#subs_classification[which(exp_aa_t_test<subs_p_cut)]<-1

	fig_path<-paste0(figFolder,"/",gene_label,"_top_subs.pdf")
#	png(fig_path, width = 6*aa_len, height = 200)
	cairo_pdf(fig_path, width = round((6*aa_len)/50,0), height = 4)
	barplot(subs_classification,xpd=FALSE,col="red",border="NA",
		  axes=F,xaxs = "i",space=0,cex.axis=2)

	axis(side=1,at=seq(0,ceiling(aa_len/20)*20,20),cex.lab=2,cex.axis=2)

	dev.off()
  
  }else{
  
  print("no significant substitution")
  }
  
  fig_path<-paste0(figFolder,"/",gene_label,"_subs_freq.pdf")
  plot_subs_freq(c(255,0,0),aa_len,avg_sum_freq,fig_path)
  
  ess<-rep(0,length(avg_freq_log))
  ess[outliers]<-1
  
  avg_sum_freq<-cbind(c(1:length(avg_sum_freq)),avg_sum_freq,exp1_sum_freq,exp2_sum_freq,subs_freq_score,subs_freq_p_value,exp_sum_cv,ess)
  
  avg_sum_freq<-avg_sum_freq[order(avg_sum_freq[,5],decreasing = T),]
  colnames(avg_sum_freq)<-c("aa_pos","subs_freq","subs_rep1","subs_rep2","subs_freq_score","subs_freq_p_value","subs_freq_CV","essentiality")
  write.table(avg_sum_freq, 
              paste0(tableFolder,"/",gene_label,"_subs_avg_freq.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = F,col.names = T) 
  
  
}





get_del_score<-function(folder,gene_label,score_cut,red_p){

  figFolder<-paste0("results_fig_",folder)
  if(!dir.exists(figFolder))dir.create(figFolder)
  tableFolder<-paste0("result_output_",folder)
  if(!dir.exists(tableFolder))dir.create(tableFolder)

	aa_seq<-as.character(unlist(read.table(paste0("aa_seq/",gene_label,"_aa.txt"),header=F,sep=",",fill=TRUE,dec=".")))
	aa_len<-nchar(aa_seq)
	WT_aa<-unlist(strsplit(aa_seq,""))

	del_score_info<-read.table(paste0(tableFolder,"/",gene_label,"_del_pos_rank.csv"),header=T,sep=",",fill=TRUE,dec=".")
	del_score_info<-del_score_info[order(del_score_info$aa_pos),]
	del_score<-del_score_info[,c(1,3)]


	#fig_path<-paste0(figFolder,"/",gene_label,"_del_score_plot.png")
	fig_path<-paste0(figFolder,"/",gene_label,"_del_score_plot.pdf")
	#red_p<-c(115,127,141)
	#black_p<-c(119,125,133,134,138)
	#red_p<-NULL
	black_p<-which(del_score[,2]>score_cut)
	#black_p<-which(del_score_info$aa_del_score>quantile(del_score_info$aa_del_score,0.975))
	plot_del_score(del_score,fig_path,red_p,black_p,WT_aa)



}

## calculate fold-change of deletion
output_del_for_subsampling<-function(folder,cds_start,cds_end,count_th,exp1_label,exp2_label,ref1_label,ref2_label,gene_label,singleDelWeight,del_cut,hist_ylim,del_len_cut){
  
  figFolder<-paste0("results_fig_",folder)
  if(!dir.exists(figFolder))dir.create(figFolder)
  tableFolder<-paste0("result_output_",folder)
  if(!dir.exists(tableFolder))dir.create(tableFolder)
  
  aa_seq<-as.character(unlist(read.table(paste0("aa_seq/",gene_label,"_aa.txt"),header=F,sep=",",fill=TRUE,dec=".")))
  aa_len<-nchar(aa_seq)
  codon_depth_exp1<-as.matrix(read.table(paste0(gene_label,"/",exp1_label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  print(mean(codon_depth_exp1))
  exp1_file<-read.table(paste0(gene_label,"/",exp1_label,".ext.sort.inframe.del.count.txt"),header=F,sep=",",fill=FALSE,skip=1,dec=".")
  #summary(as.numeric(exp1_file[,3]))
  
  ## count_th is a threshold for removing low count deletion
  exp1_file<-exp1_file[which(as.numeric(exp1_file[,3])>count_th),]
  del_codon_exp1<-convert_nt_to_codon(exp1_file,cds_start,cds_end)
  
  ### re-calculate deletion freq for deletion after count filter
  del_matrix_exp1<-del_freq_from_count(del_codon_exp1,codon_depth_exp1)
  
  del_freq_exp1<-c(as.matrix(del_matrix_exp1[,4]))

  
  codon_depth_exp2<-as.matrix(read.table(paste0(gene_label,"/",exp2_label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  print(mean(codon_depth_exp2))
  exp2_file<-read.table(paste0(gene_label,"/",exp2_label,".ext.sort.inframe.del.count.txt"),header=F,sep=",",fill=FALSE,skip=1,dec=".")
  #summary(as.numeric(exp2_file[,3]))
  
  exp2_file<-exp2_file[which(as.numeric(exp2_file[,3])>count_th),]
  del_codon_exp2<-convert_nt_to_codon(exp2_file,cds_start,cds_end)
  del_matrix_exp2<-del_freq_from_count(del_codon_exp2,codon_depth_exp2)
  del_freq_exp2<-c(as.matrix(del_matrix_exp2[,4]))
  
  
  
  codon_depth_ref1<-as.matrix(read.table(paste0(gene_label,"/",ref1_label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  print(mean(codon_depth_ref1))
  ref1_file<-read.table(paste0(gene_label,"/",ref1_label,".ext.sort.inframe.del.count.txt"),header=F,sep=",",fill=FALSE,skip=1,dec=".")
  #summary(as.numeric(ref1_file[,3]))
  
  ref1_file<-ref1_file[which(as.numeric(ref1_file[,3])>count_th),]

  del_codon_ref1<-convert_nt_to_codon(ref1_file,cds_start,cds_end)

  del_matrix_ref1<-del_freq_from_count(del_codon_ref1,codon_depth_ref1)
  del_freq_ref1<-c(as.matrix(del_matrix_ref1[,4]))

  
  
  
  codon_depth_ref2<-as.matrix(read.table(paste0(gene_label,"/",ref2_label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  print(mean(codon_depth_ref2))
  ref2_file<-read.table(paste0(gene_label,"/",ref2_label,".ext.sort.inframe.del.count.txt"),header=F,sep=",",fill=FALSE,skip=1,dec=".")
  #summary(as.numeric(ref2_file[,3]))
  
  ref2_file<-ref2_file[which(as.numeric(ref2_file[,3])>count_th),]
  del_codon_ref2<-convert_nt_to_codon(ref2_file,cds_start,cds_end)
  del_matrix_ref2<-del_freq_from_count(del_codon_ref2,codon_depth_ref2)
  del_freq_ref2<-c(as.matrix(del_matrix_ref2[,4]))

  del_freq_matrix_exp1<-construct_del_matrix(del_matrix_exp1,aa_len)
  del_freq_exp1<-c(as.matrix(del_freq_matrix_exp1))
  #summary(del_freq_exp1)
  
  del_freq_matrix_exp2<-construct_del_matrix(del_matrix_exp2,aa_len)
  del_freq_exp2<-c(as.matrix(del_freq_matrix_exp2))
  #summary(del_freq_exp2)
  
  del_freq_matrix_ref1<-construct_del_matrix(del_matrix_ref1,aa_len)
  del_freq_ref1<-c(as.matrix(del_freq_matrix_ref1))
  #summary(del_freq_ref1)
  
  del_freq_matrix_ref2<-construct_del_matrix(del_matrix_ref2,aa_len)
  del_freq_ref2<-c(as.matrix(del_freq_matrix_ref2))
  #summary(del_freq_ref2)
  

  ###########

  ## output deletion log fold change table
  
  max_del_len<-10
  del_lfc<-log2((del_freq_matrix_exp1+del_freq_matrix_exp2+1e-07)/(del_freq_matrix_ref1+del_freq_matrix_ref2+1e-07))
  summary(del_lfc)
  #del_lfc<-del_lfc[,c(1:max_del_len,ncol(del_lfc))]  
  #colnames(del_lfc)<-c(paste0("del_len_",c(1:max_del_len)),"single_aa_del")
  colnames(del_lfc)<-c(paste0("perfect_del_len_",c(1:max_del_len)),
  paste0("shift_del_len_",c(1:max_del_len)))
  write.table(del_lfc, 
              paste0(tableFolder,"/",gene_label,"_del_lfc.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = T,col.names = NA) 
  
  #for(aa in 1:nrow(del_lfc)){
  #  tmp<-del_lfc[aa,]
  #  del_lfc[aa,which(tmp<del_cut)]<-0
  #}
  
  del_fc<-(del_freq_matrix_exp1+del_freq_matrix_exp2+1e-07)/(del_freq_matrix_ref1+del_freq_matrix_ref2+1e-07)
  summary(del_fc)
  #del_fc<-del_fc[,c(1:max_del_len,ncol(del_fc))]  
  #colnames(del_fc)<-c(paste0("del_len_",c(1:max_del_len)),"single_aa_del")
  colnames(del_fc)<-c(paste0("perfect_del_len_",c(1:max_del_len)),
  paste0("shift_del_len_",c(1:max_del_len)))
  write.table(del_fc, 
              paste0(tableFolder,"/",gene_label,"_del_fc.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = T,col.names = NA) 
			  
  ## --optional--output deletion freq of reference data
  del_ref_comb<-(del_freq_matrix_ref1+del_freq_matrix_ref2)/2
  
  
  #del_ref_comb<-del_ref_comb[,c(1:max_del_len,ncol(del_ref_comb))]
  #colnames(del_ref_comb)<-c(paste0("del_len_",c(1:max_del_len)),"single_aa_del")
  colnames(del_ref_comb)<-c(paste0("perfect_del_len_",c(1:max_del_len)),
  paste0("shift_del_len_",c(1:max_del_len)))
  write.table(del_ref_comb, 
              paste0(tableFolder,"/",gene_label,"_del_ref.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = T,col.names = NA) 
			  
  ## --optional--output deletion freq of experimental data  
  del_exp_comb<-(del_freq_matrix_exp1+del_freq_matrix_exp2)/2
  #del_ref_comb<-del_ref_comb[,c(1:max_del_len,ncol(del_ref_comb))]
  #colnames(del_exp_comb)<-c(paste0("del_len_",c(1:max_del_len)),"single_aa_del")
  colnames(del_exp_comb)<-c(paste0("perfect_del_len_",c(1:max_del_len)),
  paste0("shift_del_len_",c(1:max_del_len)))
  
  
  write.table(del_exp_comb, 
              paste0(tableFolder,"/",gene_label,"_del_exp.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = T,col.names = NA) 
  
  
  
  #aa_single_del<-del_fc[,"single_aa_del"]
  aa_single_del<-del_fc[,1]
  
  if(del_len_cut>0){
    if(del_len_cut>1){
	  sum_perf_del<-get_integrated_del_sum(del_fc[,c(1:del_len_cut),drop=FALSE],1)[1:(aa_len+1)]
	  sum_shift_del<-get_shifted_del_sum(del_fc[,c(11:(del_len_cut+10)),drop=FALSE],1)[1:(aa_len+1)]
	  aa_mutiple_del<-sum_perf_del+sum_shift_del
	  }else{
	  #aa_mutiple_del<-del_fc[,1]
	  aa_mutiple_del<-del_fc[,11]
	  }
  
  }else{
  aa_mutiple_del<-rep(0,(aa_len+1))
  }
  

  plot_int_shift_del(figFolder,gene_label,sum_perf_del,sum_shift_del)
  
  
  ## calculate aa deletion strength
  ## single aa deletion gives a higher weight
  #aa_del_sum<-log2(aa_single_del*singleDelWeight+aa_mutiple_del+1)
  aa_del_sum<-log10(aa_single_del*singleDelWeight+aa_mutiple_del+1)
  #aa_del_sum<-aa_single_del*singleDelWeight+aa_mutiple_del
  
  #aa_del_z_score<-scale(aa_del_sum)
  
  ## calculate aa deletion strength p-value and essential scores 
  aa_del_p_value<-pnorm(-scale(aa_del_sum))   ### one sided p-value 
  aa_del_score<-(-log(aa_del_p_value,10))
  
  del_fc_pos<-cbind(c(1:(aa_len+1)),aa_del_sum,aa_del_score,aa_del_p_value,aa_single_del,aa_mutiple_del,sum_perf_del,sum_shift_del)
  del_fc_pos<-del_fc_pos[order(del_fc_pos[,2],decreasing = T),]
  
  colnames(del_fc_pos)<-c("aa_pos","sum_del_lfc","aa_del_score","aa_del_p_value","aa_single_del","aa_mutiple_del","perfect_del","shift_del")
  rank<-c(1:(aa_len+1))
  write.table(cbind(del_fc_pos,rank), 
              paste0(tableFolder,"/",gene_label,"_del_pos_rank.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = F,col.names = TRUE) 
  
  log_axis_cut<-del_cut

  
  fig_path<-paste0(figFolder,"/",gene_label,"_perfect_del_fc_cut")
  save_plot_del_lfc(del_fc[,1:10],aa_len,0,fig_path,3,1)
  
  fig_path<-paste0(figFolder,"/",gene_label,"_shift_del_fc_cut")
  save_plot_del_lfc(del_fc[,11:20],aa_len,0,fig_path,3,2)
  

  
  
}








## calculate fold-change of deletion
output_del<-function(folder,cds_start,cds_end,count_th,exp1_label,exp2_label,ref1_label,ref2_label,gene_label,singleDelWeight,del_cut,hist_ylim,del_len_cut){
  
  figFolder<-paste0("results_fig_",gene_label)
  if(!dir.exists(figFolder))dir.create(figFolder)
  tableFolder<-paste0("result_output_",gene_label)
  if(!dir.exists(tableFolder))dir.create(tableFolder)
  
  aa_seq<-as.character(unlist(read.table(paste0("aa_seq/",gene_label,"_aa.txt"),header=F,sep=",",fill=TRUE,dec=".")))
  aa_len<-nchar(aa_seq)
  codon_depth_exp1<-as.matrix(read.table(paste0("bam_",exp1_label,"/",exp1_label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  
  exp1_file<-read.table(paste0("bam_",exp1_label,"/",exp1_label,".ext.sort.inframe.del.count.txt"),header=F,sep=",",fill=FALSE,skip=1,dec=".")
  #summary(as.numeric(exp1_file[,3]))
  
  ## count_th is a threshold for removing low count deletion
  exp1_file<-exp1_file[which(as.numeric(exp1_file[,3])>count_th),]
  del_codon_exp1<-convert_nt_to_codon(exp1_file,cds_start,cds_end)
  
  ### re-calculate deletion freq for deletion after count filter
  del_matrix_exp1<-del_freq_from_count(del_codon_exp1,codon_depth_exp1)
  
  del_freq_exp1<-c(as.matrix(del_matrix_exp1[,4]))

  
  
  codon_depth_exp2<-as.matrix(read.table(paste0("bam_",exp2_label,"/",exp2_label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  exp2_file<-read.table(paste0("bam_",exp2_label,"/",exp2_label,".ext.sort.inframe.del.count.txt"),header=F,sep=",",fill=FALSE,skip=1,dec=".")
  #summary(as.numeric(exp2_file[,3]))
  
  exp2_file<-exp2_file[which(as.numeric(exp2_file[,3])>count_th),]
  del_codon_exp2<-convert_nt_to_codon(exp2_file,cds_start,cds_end)
  del_matrix_exp2<-del_freq_from_count(del_codon_exp2,codon_depth_exp2)
  del_freq_exp2<-c(as.matrix(del_matrix_exp2[,4]))
  
   
  
  codon_depth_ref1<-as.matrix(read.table(paste0("bam_",ref1_label,"/",ref1_label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  ref1_file<-read.table(paste0("bam_",ref1_label,"/",ref1_label,".ext.sort.inframe.del.count.txt"),header=F,sep=",",fill=FALSE,skip=1,dec=".")
  #summary(as.numeric(ref1_file[,3]))
  
  ref1_file<-ref1_file[which(as.numeric(ref1_file[,3])>count_th),]
  del_codon_ref1<-convert_nt_to_codon(ref1_file,cds_start,cds_end)
  del_matrix_ref1<-del_freq_from_count(del_codon_ref1,codon_depth_ref1)
  del_freq_ref1<-c(as.matrix(del_matrix_ref1[,4]))

  
  codon_depth_ref2<-as.matrix(read.table(paste0("bam_",ref2_label,"/",ref2_label,".coverage.codon.depth.txt"), header=TRUE,sep=","))
  ref2_file<-read.table(paste0("bam_",ref2_label,"/",ref2_label,".ext.sort.inframe.del.count.txt"),header=F,sep=",",fill=FALSE,skip=1,dec=".")
  #summary(as.numeric(ref2_file[,3]))
  
  ref2_file<-ref2_file[which(as.numeric(ref2_file[,3])>count_th),]
  del_codon_ref2<-convert_nt_to_codon(ref2_file,cds_start,cds_end)
  del_matrix_ref2<-del_freq_from_count(del_codon_ref2,codon_depth_ref2)
  del_freq_ref2<-c(as.matrix(del_matrix_ref2[,4]))

  del_freq_matrix_exp1<-construct_del_matrix(del_matrix_exp1,aa_len)
  del_freq_exp1<-c(as.matrix(del_freq_matrix_exp1))
  #summary(del_freq_exp1)
  
  del_freq_matrix_exp2<-construct_del_matrix(del_matrix_exp2,aa_len)
  del_freq_exp2<-c(as.matrix(del_freq_matrix_exp2))
  #summary(del_freq_exp2)
  
  del_freq_matrix_ref1<-construct_del_matrix(del_matrix_ref1,aa_len)
  del_freq_ref1<-c(as.matrix(del_freq_matrix_ref1))
  #summary(del_freq_ref1)
  
  del_freq_matrix_ref2<-construct_del_matrix(del_matrix_ref2,aa_len)
  del_freq_ref2<-c(as.matrix(del_freq_matrix_ref2))
  #summary(del_freq_ref2)
  
  
    combo_aa_freq_ref<-c((as.matrix(del_freq_matrix_ref1[11:20,])+as.matrix(del_freq_matrix_ref2[11:20,]))/2)
  
  
  #summary(del_freq_wt)
  
  lim_value<-ceiling(log10(max(del_freq_ref1,del_freq_ref2)+1e-07))

  

  ## output deletion log fold change table
  
  max_del_len<-10
  del_lfc<-log2((del_freq_matrix_exp1+del_freq_matrix_exp2+1e-07)/(del_freq_matrix_ref1+del_freq_matrix_ref2+1e-07))
  summary(del_lfc)
  #del_lfc<-del_lfc[,c(1:max_del_len,ncol(del_lfc))]  
  #colnames(del_lfc)<-c(paste0("del_len_",c(1:max_del_len)),"single_aa_del")
  colnames(del_lfc)<-c(paste0("perfect_del_len_",c(1:max_del_len)),
  paste0("shift_del_len_",c(1:max_del_len)))
  write.table(del_lfc, 
              paste0(tableFolder,"/",gene_label,"_del_lfc.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = T,col.names = NA) 
  

  
  del_fc<-(del_freq_matrix_exp1+del_freq_matrix_exp2+1e-07)/(del_freq_matrix_ref1+del_freq_matrix_ref2+1e-07)
  summary(del_fc)
  #del_fc<-del_fc[,c(1:max_del_len,ncol(del_fc))]  
  #colnames(del_fc)<-c(paste0("del_len_",c(1:max_del_len)),"single_aa_del")
  colnames(del_fc)<-c(paste0("perfect_del_len_",c(1:max_del_len)),
  paste0("shift_del_len_",c(1:max_del_len)))
  write.table(del_fc, 
              paste0(tableFolder,"/",gene_label,"_del_fc.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = T,col.names = NA) 
			  

  
  
  
  #aa_single_del<-del_fc[,"single_aa_del"]
  aa_single_del<-del_fc[,1]
  
  if(del_len_cut>0){
    if(del_len_cut>1){
	  sum_perf_del<-get_integrated_del_sum(del_fc[,c(1:del_len_cut),drop=FALSE],1)[1:(aa_len+1)]
	  sum_shift_del<-get_shifted_del_sum(del_fc[,c(11:(del_len_cut+10)),drop=FALSE],1)[1:(aa_len+1)]
	  aa_mutiple_del<-sum_perf_del+sum_shift_del
	  }else{
	  #aa_mutiple_del<-del_fc[,1]
	  aa_mutiple_del<-del_fc[,11]
	  }
  
  }else{
  aa_mutiple_del<-rep(0,(aa_len+1))
  }
  

  #plot_int_shift_del(figFolder,gene_label,sum_perf_del,sum_shift_del)
  
  
  ## calculate aa deletion strength
  ## single aa deletion gives a higher weight
  #aa_del_sum<-log2(aa_single_del*singleDelWeight+aa_mutiple_del+1)
  aa_del_sum<-log10(aa_single_del*singleDelWeight+aa_mutiple_del+1)
  #aa_del_sum<-aa_single_del*singleDelWeight+aa_mutiple_del
  
  #aa_del_z_score<-scale(aa_del_sum)
  
  ## calculate aa deletion strength p-value and essential scores 
  aa_del_p_value<-pnorm(-scale(aa_del_sum))   ### one sided p-value 
  aa_del_score<-(-log(aa_del_p_value,10))
  
  del_fc_pos<-cbind(c(1:(aa_len+1)),aa_del_sum,aa_del_score,aa_del_p_value,aa_single_del,aa_mutiple_del,sum_perf_del,sum_shift_del)
  del_fc_pos<-del_fc_pos[order(del_fc_pos[,2],decreasing = T),]
  
  colnames(del_fc_pos)<-c("aa_pos","sum_del_lfc","aa_del_score","aa_del_p_value","aa_single_del","aa_mutiple_del","perfect_del","shift_del")
  rank<-c(1:(aa_len+1))
  write.table(cbind(del_fc_pos,rank), 
              paste0(tableFolder,"/",gene_label,"_del_pos_rank.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = F,col.names = TRUE) 
  
  log_axis_cut<-del_cut
  #plot_del_lfc(del_lfc,aa_len,del_cut)
  #fig_path<-paste0(figFolder,"/",gene_label,"_perfect_del_lfc_cut")
  #save_plot_del_lfc(del_lfc[,1:10],aa_len,log_axis_cut,fig_path,3,1)
  
  #fig_path<-paste0(figFolder,"/",gene_label,"_shift_del_lfc_cut")
  #save_plot_del_lfc(del_lfc[,11:20],aa_len,log_axis_cut,fig_path,3,2)
  
  fig_path<-paste0(figFolder,"/",gene_label,"_perfect_del_fc_cut")
  save_plot_del_lfc(del_fc[,1:10],aa_len,0,fig_path,3,1)
  
  fig_path<-paste0(figFolder,"/",gene_label,"_shift_del_fc_cut")
  save_plot_del_lfc(del_fc[,11:20],aa_len,0,fig_path,3,2)
  
  #fig_path<-paste0(figFolder,"/",gene_label,"_double_del_fc_cut")
  #save_plot_del_lfc_two_type(del_fc,aa_len,0,fig_path,10)
  
  #fig_path<-paste0(figFolder,"/",gene_label,"_double_del_lfc_cut")
  #save_plot_del_lfc_two_type(del_lfc,aa_len,log_axis_cut,fig_path,10)
  
  
}



## output substituion info
output_subs<-function(folder,cds_start,cds_end,count_th,exp1_label,exp2_label,ref1_label,ref2_label,gene_label){
  
  figFolder<-paste0("results_fig_",gene_label)
  if(!dir.exists(figFolder))dir.create(figFolder)
  tableFolder<-paste0("result_output_",gene_label)
  if(!dir.exists(tableFolder))dir.create(tableFolder)
  
  aa_seq<-as.character(unlist(read.table(paste0("aa_seq/",gene_label,"_aa.txt"),header=F,sep=",",fill=TRUE,dec=".")))

  aa_len<-nchar(aa_seq)
  
  subs_exp1_freq<-subs_count_cut_freq(paste0("bam_",exp1_label),exp1_label,count_th) 
  subs_exp2_freq<-subs_count_cut_freq(paste0("bam_",exp2_label),exp2_label,count_th)
 
  subs_ref1_freq<-subs_count_cut_freq(paste0("bam_",ref1_label),ref1_label,count_th)  
  subs_ref2_freq<-subs_count_cut_freq(paste0("bam_",ref2_label),ref2_label,count_th)


  
  ### remove SNP 
  SNP_cut<-0.1
  tmp<-which(subs_ref1_freq>SNP_cut & subs_ref2_freq>SNP_cut)
  if(length(tmp)>0){
  print("SNP")
  print(tmp)
	  subs_exp1_freq[subs_ref1_freq>SNP_cut & subs_ref2_freq>SNP_cut]<-0
	  subs_exp2_freq[subs_ref1_freq>SNP_cut & subs_ref2_freq>SNP_cut]<-0
	  subs_ref1_freq[subs_ref1_freq>SNP_cut & subs_ref2_freq>SNP_cut]<-0
	  subs_ref2_freq[subs_ref1_freq>SNP_cut & subs_ref2_freq>SNP_cut]<-0

  }
  
	
  aa_subs_exp1_freq<-codonToAA(subs_exp1_freq)
  aa_subs_exp2_freq<-codonToAA(subs_exp2_freq)
  aa_subs_ref1_freq<-codonToAA(subs_ref1_freq)
  aa_subs_ref2_freq<-codonToAA(subs_ref2_freq)

    
  exp1_sum_freq<-colSums(aa_subs_exp1_freq)
  exp2_sum_freq<-colSums(aa_subs_exp2_freq)
  
  ref1_sum_freq<-colSums(aa_subs_ref1_freq)
  ref2_sum_freq<-colSums(aa_subs_ref2_freq)
  
  #range(avg_sum_freq)
  
  aa_ref_freq<-(aa_subs_ref1_freq+aa_subs_ref2_freq)/2
  
  
  
  exp_aa_t_test<-rep(1,length(exp1_sum_freq))
  
  exp1_sum_log_freq<-log10(exp1_sum_freq+1e-05)
  exp2_sum_log_freq<-log10(exp2_sum_freq+1e-05)
  
  ref1_sum_log_freq<-log10(ref1_sum_freq+1e-05)
  ref2_sum_log_freq<-log10(ref2_sum_freq+1e-05)
  
#  for(tt in 1:length(exp_aa_t_test)){
#    exp_aa_t_test[tt]<-t.test(c(exp1_sum_log_freq[tt],exp2_sum_log_freq[tt]),
#			c(ref1_sum_log_freq[tt],ref2_sum_log_freq[tt]),alternative="greater")$p.value	
  #exp_aa_t_test[tt]<-t.test(c(exp1_sum_freq[tt],exp2_sum_freq[tt]),
	#		c(ref1_sum_freq[tt],ref2_sum_freq[tt]),alternative="greater")$p.value			
#  }

  
  avg_sum_freq<-(exp1_sum_freq+exp2_sum_freq)/2
  
  exp_sum_cv<-as.matrix(unlist(apply(cbind(exp1_sum_freq,exp2_sum_freq),1,cv)))
  #summary(avg_sum_freq)
  
  ##  calculate p-value and essential score of substitution
  
  set.seed(90)
  neg_dist<-rnorm(100)
   
  exp1_sum_freq_z<-scale(exp1_sum_freq)
  exp2_sum_freq_z<-scale(exp2_sum_freq)  
  
  #subs_freq_p_value<-pnorm(-scale(log10(avg_sum_freq+1e-05)))
  
  subs_freq_p_value<-rep(1,length(exp1_sum_freq_z))
  for(zz in 1:length(exp1_sum_freq_z)){
    #subs_freq_p_value[zz]<-t.test(x=c(exp1_sum_freq_z[zz],exp2_sum_freq_z[zz]),y=neg_dist, alternative ="greater")$p.value
	subs_freq_p_value[zz]<-wilcox.test(x=c(exp1_sum_freq_z[zz],exp2_sum_freq_z[zz]),y=neg_dist, alternative ="greater")$p.value
  
  }

  #subs_freq_p_value<-pnorm(-scale(avg_sum_freq))
  
  #print(subs_freq_p_value)
  subs_freq_score<-(-log10(subs_freq_p_value+1e-05))+log10(avg_sum_freq+1e-05)+10
  #is.na(subs_freq_score)==0
  subs_freq_score<-(subs_freq_score-min(subs_freq_score))/(max(subs_freq_score)-min(subs_freq_score))
  
  subs_classification<-rep(0,length(avg_sum_freq))
  
  ## cutsubstituion freq 
  #subs_p_cut<-0.05
  avg_freq_log<-log10(avg_sum_freq+1e-05)
  subs_cut<-mean(avg_freq_log)+3*sd(avg_freq_log)

  
  
  ########
  
  outliers<-which(avg_freq_log>subs_cut)
  
  if(length(outliers)>0){
  
  print(outliers)

  }else{
  
  print("no significant substitution")
  }
  



  fig_path<-paste0(figFolder,"/",gene_label,"_subs_freq.pdf")
  plot_subs_freq(c(255,0,0),aa_len,avg_sum_freq,fig_path)
  
  
  avg_sum_freq<-cbind(c(1:length(avg_sum_freq)),avg_sum_freq,exp1_sum_freq,exp2_sum_freq,subs_freq_score,subs_freq_p_value,exp_sum_cv)
  avg_sum_freq<-cbind(avg_sum_freq[order(avg_sum_freq[,5],decreasing = T),],c(1:nrow(avg_sum_freq)))
  colnames(avg_sum_freq)<-c("aa_pos","subs_freq","subs_rep1","subs_rep2","subs_freq_score","subs_freq_p_value","subs_freq_CV","rank")
  write.table(avg_sum_freq, 
              paste0(tableFolder,"/",gene_label,"_subs_avg_freq.csv"), 
              append = FALSE, quote = FALSE, sep = ",",
              row.names = F,col.names = T) 
  
  
}







